"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTsconfigFilePath = getTsconfigFilePath;
exports.loadTsconfig = loadTsconfig;
const node_path_1 = __importDefault(require("node:path"));
const node_core_library_1 = require("@rushstack/node-core-library");
/**
 * @internal
 */
function getTsconfigFilePath(heftConfiguration, tsconfigRelativePath) {
    return node_core_library_1.Path.convertToSlashes(
    // Use path.resolve because the path can start with `./` or `../`
    node_path_1.default.resolve(heftConfiguration.buildFolderPath, tsconfigRelativePath !== null && tsconfigRelativePath !== void 0 ? tsconfigRelativePath : './tsconfig.json'));
}
/**
 * @internal
 */
function loadTsconfig(options) {
    const { tool: { ts, system }, tsconfigPath, tsCacheFilePath } = options;
    const parsedConfigFile = ts.readConfigFile(tsconfigPath, system.readFile);
    const currentFolder = node_path_1.default.dirname(tsconfigPath);
    const tsconfig = ts.parseJsonConfigFileContent(parsedConfigFile.config, {
        fileExists: system.fileExists,
        readFile: system.readFile,
        readDirectory: system.readDirectory,
        realpath: system.realpath,
        useCaseSensitiveFileNames: true
    }, currentFolder, 
    /*existingOptions:*/ undefined, tsconfigPath);
    if (tsconfig.options.incremental) {
        tsconfig.options.tsBuildInfoFile = tsCacheFilePath;
    }
    return tsconfig;
}
//# sourceMappingURL=tsconfigLoader.js.map